from Tkinter import *

class FormDialog( Frame ):
    """
    """
    def __init__( self, parent, labels_and_vars ):

        Frame.__init__( self, parent )

        self._org_vars = []
        self._working_vars = []
        self._makeWidgets( labels_and_vars )

    def _makeLabeledEntry( self, frame, row, label_text, linkvar ):

        l = Label( frame, text=label_text )
        l.grid( row=row, col=0, sticky="w" )

        e = Entry( frame, textvariable=linkvar )
        e.grid( row=row, col=1, sticky="we" )

        return e

    def _makeLabeledCheckbox( self, frame, row, label_text, linkvar ):

        l = Label( frame, text=label_text )
        l.grid( row=row, col=0, sticky="w" )

        e = Checkbutton( frame, variable=linkvar )
        e.grid( row=row, col=1, sticky="w" )

        return e

    def _makeButton( self, bar, caption, command ):

        b = Button( bar, text=caption, command=command )
        b.pack( side=LEFT )

        return b

    def _makeWidgets( self, labels_and_vars ):

        self.pack()

        frame = Frame( self )
        frame.grid_columnconfigure( 0, weight=0 )
        frame.grid_columnconfigure( 1, weight=100 )

        row = 0
        for label, var in labels_and_vars:

            self._org_vars.append( var )
            new_var = var.__class__()
            new_var.set( var.get() )
            self._working_vars.append( new_var )

            if var.__class__ is BooleanVar:
                self._makeLabeledCheckbox( frame, row, label, new_var )
            else:
                self._makeLabeledEntry( frame, row, label, new_var )

            row = row + 1

        frame.pack( side=TOP )

        bar = Frame( self )

        self._makeButton( bar, "OK", self._ok )
        self._makeButton( bar, "Cancel", self.master.destroy )

        bar.pack( side=BOTTOM )

    def _ok( self ):
        """
            Copy working vars back to originals.
        """
        for i in range( len ( self._working_vars ) ):
            self._org_vars[i].set( self._working_vars[i].get() )

        self.master.destroy()

class TkLoggingProxy( Frame ):

    def __init__( self, parent=None ):

        Frame.__init__( self, parent )

        self._verbosity         = IntVar()
        self._listen_host       = StringVar()
        self._listen_port       = IntVar()
        self._logfile_directory = StringVar()
        self._logfile_prefix    = StringVar()
        self._record_inbound    = BooleanVar()

        self._makeWidgets()

        self._verbosity.set( 2 )
        self._listen_host.set( 'localhost' )
        self._listen_port.set( 9999 )
        self._logfile_directory.set( '/tmp' )
        self._logfile_prefix.set( 'zft' )
        self._record_inbound.set( 0 )

        self.master.title( "Logging HTTP Proxy" )

    def _makeLabeledEntry( self, frame, row, label_text, linkvar ):

        l = Label( frame, text=label_text )
        l.grid( row=row, col=0, sticky="w" )

        e = Entry( frame, textvariable=linkvar )
        e.grid( row=row, col=1, sticky="we" )

        return e

    def _makeLabeledCheckbox( self, frame, row, label_text, linkvar ):

        l = Label( frame, text=label_text )
        l.grid( row=row, col=0, sticky="w" )

        e = Checkbutton( frame, variable=linkvar )
        e.grid( row=row, col=1, sticky="w" )

        return e

    def _makeButton( self, bar, caption, command ):

        b = Button( bar, text=caption, command=command )
        b.pack( side=LEFT )

        return b

    def _showOptions( self ):

        self._showLogMessage( 'Listen: %s:%d'
                            % ( self._listen_host.get()
                              , self._listen_port.get()
                              )
                            )

        self._showLogMessage( 'Logging: %s, %s'
                            % ( self._logfile_directory.get()
                              , self._logfile_prefix.get()
                              )
                            )

    def _optionsDialog( self, labels_and_vars ):

        top = Toplevel()
        dialog = FormDialog( top, labels_and_vars )
        top.grab_set()
        top.focus_set()
        top.wait_window()
        
    def _serverOptions( self ):

        self._optionsDialog( ( ( "Host", self._listen_host )
                             , ( "Port", self._listen_port )
                             )
                           )

    def _loggingOptions( self ):

        self._optionsDialog( ( ( "Directory", self._logfile_directory )
                             , ( "Prefix", self._logfile_prefix )
                             )
                           )

    def _showLogMessage( self, message ):
        self._text.configure( state=NORMAL )
        try:
            self._text.insert( END, '%s\n' % message )
        finally:
            self._text.configure( state=DISABLED )
        self._text.see( END )

    def _makeWidgets( self ):
        
        self.pack()

        bar = Frame( self )
        self._makeButton( bar, "Server Options", self._serverOptions )
        self._makeButton( bar, "Logging Options", self._loggingOptions )
        self._makeButton( bar, "Check", self._showOptions )

        bar.pack( side=TOP )

        frame = Frame( self )
        vbar = Scrollbar( frame, name='vbar' )
        text = self._text = Text( frame, name='text', padx=5
                                , background="white", wrap="none" )

        vbar[ 'command' ] = text.yview
        vbar.pack( side=RIGHT, fill=Y )

        text[ 'yscrollcommand' ] = vbar.set
        text.configure( state=DISABLED )
        text.pack( side=LEFT, fill=BOTH, expand=1 )

        frame.pack( fill=BOTH, expand=1 )

        self._showLogMessage( "We're up!" )


if __name__ == '__main__':

    main_window = TkLoggingProxy()
    main_window.mainloop()
