import unittest


class CPETests( unittest.TestCase ):

    def _makeOne( self, *args, **kw ):
        from FunctionalTests.ConfigParserExt import ConfigParserExt
        return ConfigParserExt( *args, **kw )

    def test_empty( self ):

        empty = self._makeOne()

        self.assertEquals( empty.listSpecialSections(), ( 'DEFAULT', ) )
        self.failIf( empty.listOtherSections() )

    def _getTestPath( self, *args ):
        import os
        import FunctionalTests
        package_path = os.path.abspath( FunctionalTests.__path__[0] )
        return os.path.join( package_path, *args )

    def test_withFile( self ):

        cpe = self._makeOne( ( 'Header', 'DEFAULT' ) )

        self.assertEquals( len( cpe.listSpecialSections() ), 2 )
        self.failUnless( 'Header' in cpe.listSpecialSections() )
        self.failUnless( 'DEFAULT' in cpe.listSpecialSections() )
        self.failIf( cpe.listOtherSections() )

        filename =  self._getTestPath( 'tests', 'config.ini' )
        cpe.read( filename )

        self.assertEquals( len( cpe.listOtherSections() ), 5 )

        for i in range( 6 )[1:]:
            section = 'Section_%d' % i
            self.failUnless( section in cpe.listOtherSections() )
            self.assertEquals( cpe.get( section, 'key' ), section.lower() )


def test_suite():
    return unittest.makeSuite( CPETests )

if __name__ == '__main__':
    unittest.main()
