import unittest



class FauxInvocation:

    def __init__( self, result, elapsed ):
        self._result    = result
        self._elapsed   = elapsed

    def getResult( self ):
        return self._result

    def getElapsedTime( self ):
        return self._elapsed

class FauxRequest:

    def __init__( self, name, result, time ):
        self._name      = name
        self._result    = result
        self._time      = time

    def getName( self ):
        return self._name

    def getExpectedResult( self ):
        return self._result

    def getExpectedTime( self ):
        return self._time

MARKER = []
MARKER2 = []
class ResultTests( unittest.TestCase ):

    def _makeOne( self, *args, **kw ):
        from FunctionalTests.Framework import Result
        return Result( *args, **kw )

    def test_empty( self ):

        empty = self._makeOne( test=MARKER )

        self.failUnless( empty.getTest() is MARKER )
        self.failUnless( empty.timeRequests() )
        self.failUnless( empty.checkResponses() )
        self.failUnless( empty.checkElapsedTimes() )
        self.failIf( empty.listInvocations() )
        self.failIf( empty.listErrors() )
        self.failIf( empty.listErrors( fatal_only=1 ) )
        self.failIf( empty.listChildren() )

    def test_apathy( self ):

        apathy = self._makeOne( test=MARKER
                              , time_requests=0
                              , check_responses=0
                              , check_elapsed_times=0
                              )

        self.failUnless( apathy.getTest() is MARKER )
        self.failIf( apathy.timeRequests() )
        self.failIf( apathy.checkResponses() )
        self.failIf( apathy.checkElapsedTimes() )

        apathy.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( apathy.listInvocations() ), 1 )
        self.failIf( apathy.listErrors() )
        self.failIf( apathy.listErrors( fatal_only=1 ) )
        self.failIf( apathy.listChildren() )

    def test_responseOnly( self ):

        driven = self._makeOne( test=MARKER
                              , time_requests=0
                              , check_responses=1
                              , check_elapsed_times=0
                              )

        self.failUnless( driven.getTest() is MARKER )
        self.failIf( driven.timeRequests() )
        self.failUnless( driven.checkResponses() )
        self.failIf( driven.checkElapsedTimes() )

        driven.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( driven.listInvocations() ), 1 )
        self.assertEquals( len( driven.listErrors() ), 1 )
        self.assertEquals( len( driven.listErrors( fatal_only=1 ) ), 1 )
        self.failIf( driven.listChildren() )

    def test_timeOnly( self ):

        consistent = self._makeOne( test=MARKER
                                  , time_requests=0
                                  , check_responses=0
                                  , check_elapsed_times=1
                                  )

        self.failIf( consistent.timeRequests() )

        really = self._makeOne( test=MARKER
                              , time_requests=1
                              , check_responses=0
                              , check_elapsed_times=1
                              )

        self.failUnless( really.timeRequests() )

        impatiens = self._makeOne( test=MARKER
                                 , time_requests=1
                                 , check_responses=0
                                 , check_elapsed_times=1
                                 )

        self.failUnless( impatiens.getTest() is MARKER )
        self.failUnless( impatiens.timeRequests() )
        self.failIf( impatiens.checkResponses() )
        self.failUnless( impatiens.checkElapsedTimes() )

        impatiens.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( impatiens.listInvocations() ), 1 )
        self.assertEquals( len( impatiens.listErrors() ), 1 )
        self.failIf( impatiens.listErrors( fatal_only=1 ) )
        self.failIf( impatiens.listChildren() )

    def test_composite( self ):

        top = self._makeOne( test=MARKER )

        self.failUnless( top.getTest() is MARKER )
        self.failUnless( top.timeRequests() )
        self.failUnless( top.checkResponses() )
        self.failUnless( top.checkElapsedTimes() )

        child = top.newChild( test=MARKER2 )

        self.failUnless( child.getTest() is MARKER2 )
        self.assertEquals( child.timeRequests(), top.timeRequests() )
        self.assertEquals( child.checkResponses(), top.checkResponses() )
        self.assertEquals( child.checkElapsedTimes(), top.checkElapsedTimes() )

        child.addInvocation( FauxInvocation( 500, 3600.0 )
                            , FauxRequest( 'foo', 200, 1.0 )
                            )
        self.assertEquals( len( child.listInvocations() ), 1 )
        self.assertEquals( len( child.listErrors() ), 1 )
        self.assertEquals( len( child.listErrors( fatal_only=1 ) ), 1 )

        self.assertEquals( len( top.listInvocations() ), 1 )
        self.assertEquals( len( top.listInvocations( roll_up=0 ) ), 0 )
        self.assertEquals( len( top.listErrors() ), 1 )
        self.assertEquals( len( top.listErrors( roll_up=0 ) ), 0 )
        self.assertEquals( len( top.listErrors( fatal_only=1 ) ), 1 )
        self.assertEquals( len( top.listErrors( fatal_only=1, roll_up=0 ) ), 0 )
        self.assertEquals( len( top.listChildren() ), 1 )

        top = self._makeOne( test=MARKER
                           , application=[]
                           , defaults={ 'foo':0 }
                           , time_requests=0
                           , check_responses=0
                           , check_elapsed_times=0
                           )
        child = top.newChild( test=MARKER2 )
        self.failUnless( child.getApplication() is top.getApplication() )
        self.assertEquals( child.getDefaults(), top.getDefaults() )
        self.assertEquals( child.timeRequests(), top.timeRequests() )
        self.assertEquals( child.checkResponses(), top.checkResponses() )
        self.assertEquals( child.checkElapsedTimes(), top.checkElapsedTimes() )


def test_suite():
    return unittest.makeSuite( ResultTests )

if __name__ == '__main__':
    unittest.main()
